/*
rus2lat

Cyrillic script to Latin script (translit) conversion.
Latin to cyrillic is also supported, but not very well yet.

Copyright (C) 2008 by Kitya Karlson http://www.karlson.ru/firefox/

Tested with Firefox 3.0

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation in version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*! \mainpage rus2lat
 *
 * \section intro Introduction
 * <p>Cyrillic script to Latin script (translit) conversion.
 * Latin to cyrillic is also supported, but not very well yet.
 *
 * <p>Addon page: <a href="https://addons.mozilla.org/en-US/firefox/addon/797">https://addons.mozilla.org/en-US/firefox/addon/797</a>
 *
 * <p>Tested with Firefox versions 2 and 3
 * 
 * \section downloads Downloads
 * <ul>
 * <li><a href="../rus2lat.xpi">Download & install the latest version.</a>
 * </ul>
 *
 * \section changes Changes
 * <p><b>v 0.4</b>
 * <ul>
 * <li>Updated for firefox 3.1b2.
 * </ul>
 * <p><b>v 0.3</b>
 * <ul>
 * <li>Updated for firefox 3.0.
 * <li>Support for selection in text input fields.
 * </ul>
 * <p><b>v 0.2</b>
 * <ul>
 * <li>Updated for firefox 2.0.
 * </ul>
 * <p><b>v 0.1</b>
 * <ul>
 * <li>Initial release.
 * </ul>
 *
 * \section legal Legal
 * <p>Copyright (C) 2008 by Kitya Karlson, <a href="http://www.karlson.ru/firefox/">http://www.karlson.ru/firefox/</a>.
 * <p>This software is distributed under <a href="http://www.gnu.org/licenses/gpl.html">GPL</a>.
 *
 */

/**
 * @file
 * Cyrillic script to Latin script (translit) and translit to Cyrillic script conversions are done by this JavaScript program.
 */

/**
 * Latin table for Cyrillic to Latin script translation.
 * @type Array
 */
var LatinScript = new Array("A", "B", "V", "G", "D", "E", "E", "ZH", "Z", "I", "J", "K", "L", "M", "N", "O", "P", "R", "S", "T", "U", "F", "H", "C", "CH", "SH", "SCH", "\"", "Y", "'", "E", "IU", "IA", "a", "b", "v", "g", "d", "e", "e", "zh", "z", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u", "f", "h", "c", "ch", "sh", "sch", "\"", "y", "'", "e", "iu", "ia");
/**
 * Latin table for Latin to Cyrillic script translation.
 * @type Array
 */
var LatinPhoneticScript = Array("fonetics", "recycle", "cation", "lution", "mation", "clean", "nctio", "sheme", "aded", "date", "file", "iles", "kill", "line", "lite", "name", "page", "rces", "time", "ware", "ace", "ade", "ake", "all", "cut", "dec", "ere", "exa", "hew", "hoy", "ice", "iew", "ile", "isu", "ity", "ive", "les", "lib", "mee", "mes", "nes", "net", "new", "new", "one", "rce", "sch", "tle", "tun", "uil", "ume", "ca", "ch", "ck", "co", "cr", "cs", "ct", "cu", "ea", "ee", "es", "eu", "ge", "gg", "gh", "gi", "ha", "he", "hi", "ho", "hu", "hu", "hy", "il", "im", "li", "ly", "me", "mu", "oo", "ow", "oy", "ph", "sh", "sm", "th", "ty", "us", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z");
/**
 * Cyrillic (UTF-8) table for Cyrillic to Latin script translation.
 * @type Array
 */
var CyrillicScript = new Array("\u0410", "\u0411", "\u0412", "\u0413", "\u0414", "\u0415", "\u0401", "\u0416", "\u0417", "\u0418", "\u0419", "\u041A", "\u041B", "\u041C", "\u041D", "\u041E", "\u041F", "\u0420", "\u0421", "\u0422", "\u0423", "\u0424", "\u0425", "\u0426", "\u0427", "\u0428", "\u0429", "\u042A", "\u042B", "\u042C", "\u042D", "\u042E", "\u042F", "\u0430", "\u0431", "\u0432", "\u0433", "\u0434", "\u0435", "\u0451", "\u0436", "\u0437", "\u0438", "\u0439", "\u043A", "\u043B", "\u043C", "\u043D", "\u043E", "\u043F", "\u0440", "\u0441", "\u0442", "\u0443", "\u0444", "\u0445", "\u0446", "\u0447", "\u0448", "\u0449", "\u044A", "\u044B", "\u044C", "\u044D", "\u044E", "\u044F");
/**
 * Cyrillic (UTF-8) table for Latin to Cyrillic script translation.
 * @type Array
 */
var CyrillicPhoneticScript = Array("\u0444\u043E\u043D\u0435\u0442\u0438\u043A\u0441", "\u0440\u0435\u043A\u0430\u0439\u043A\u043B", "\u043A\u044D\u0439\u0448\u0435\u043D", "\u043B\u044E\u0448\u043D", "\u043C\u0435\u0439\u0448\u0435\u043D", "\u043A\u043B\u0438\u043D", "\u043D\u043A\u0448\u0435", "\u0441\u0445\u0438\u043C", "\u0430\u0434\u0438\u0434", "\u0434\u0435\u0439\u0442", "\u0444\u0430\u0439\u043B", "\u0430\u0439\u043B\u0437", "\u043A\u0438\u043B", "\u043B\u0430\u0439\u043D", "\u043B\u0430\u0439\u0442", "\u043D\u0435\u0439\u043C", "\u043F\u0435\u0439\u0434\u0436", "\u0440\u0441\u0438\u0437", "\u0442\u0430\u0439\u043C", "\u0432\u0435\u0439\u0440", "\u044D\u0439\u0441", "\u044D\u0439\u0434", "\u0430\u0439\u043A", "\u043E\u043B\u043B", "\u043A\u0430\u0442", "\u0434\u0435\u043A", "\u0435\u0440", "\u0435\u043A\u0437", "\u0445\u044C\u044E", "\u0445\u043E\u0439", "\u0430\u0439\u0441", "\u044C\u044E\u0432", "\u0430\u0439\u043B", "\u0438\u0436\u0443", "\u0438\u0442\u0438", "\u0430\u0439\u0432", "\u043B\u0441", "\u043B\u0430\u0439\u0431", "\u043C\u0438", "\u043C\u0437", "\u043D\u0441", "\u043D\u0435\u0442", "\u043D\u044C\u044E", "\u043D\u044C\u044E", "\u043E\u043D", "\u0440\u0441", "\u0449", "\u0442\u043B", "\u0442\u044E\u043D", "\u0438\u043B", "\u044E\u043C", "\u043A\u0430", "\u0447", "\u043A", "\u043A\u043E", "\u043A\u0440", "\u043A", "\u043A\u0442", "\u043A\u0443", "\u0438", "\u0438", "\u0438\u0437", "\u0435\u0432", "\u0434\u0436\u0435", "\u0433\u0433", "", "\u0434\u0436\u0438", "\u0445\u0430\u0439", "\u0445\u0435", "\u0445\u0430\u0439", "\u0445\u043E", "\u0445\u0443", "\u0445\u044C\u044E", "\u0445\u0438", "\u0430\u0439\u043B", "\u0430\u0439\u043C", "\u043B\u0430\u0439", "\u043B\u0430\u0439", "\u043C\u0438", "\u043C\u044C\u044E", "\u0443", "\u0430\u0443", "\u043E\u0439", "\u0444", "\u0448", "\u0437\u043C", "\u0437", "\u0442\u0430\u0439", "\u044E\u0437", "\u0410", "\u0411", "\u0421", "\u0414", "\u0415", "\u0444", "\u0413", "\u0428", "\u0418", "\u0414\u0436", "\u041A", "\u041B", "\u041C", "\u041D", "\u041E", "\u041F", "\u041A", "\u0420", "\u0421", "\u0422", "\u0423", "\u0412", "\u0412", "\u041A\u0441", "\u0418", "\u0417", "\u0430", "\u0431", "\u0441", "\u0434", "\u0435", "\u0444", "\u0433", "\u0448", "\u0438", "\u0434\u0436", "\u043A", "\u043B", "\u043C", "\u043D", "\u043E", "\u043F", "\u043A", "\u0440", "\u0441", "\u0442", "\u0443", "\u0432", "\u0432", "\u043A\u0441", "\u0438", "\u0437");


/**
 * Convertor class provides functions for script conversion
 *
 * @ctor
 * Constructs Convertor class (empty).
 */
function Convertor() {
};

/**
 * Helper function. Converts single cyrillic character to one or more latin characters.
 * @tparam String ch Cyrillic single character to convert.
 * @treturn String Corresponding Latin character(s).
 */
Convertor.prototype.Rus2LatChar = function (ch) {
	for (var i in CyrillicScript) {
		if (ch == CyrillicScript[i]) {
			return LatinScript[i];
		}
	}
	return ch;
};

/**
 * Converts text in Cyrillic script to Latin script.
 * @tparam String text Cyrillic text to convert.
 * @treturn String Corresponding Latin text.
 */
Convertor.prototype.Rus2LatText = function(text) {
	var result = new String("");
	for (j = 0; j < text.length; j++) {
		result += this.Rus2LatChar(text.charAt(j));
	}
	return result;
};
/**
 * Converts text in Latin script to Cyrillic script.
 * @tparam String text Latin text to convert.
 * @treturn String Corresponding Cyrillic text.
 */
Convertor.prototype.Lat2RusText = function(text) {
	var reg;
	for (var i in LatinPhoneticScript) {
		if (LatinPhoneticScript[i].length == 1) {
			reg = new RegExp(LatinPhoneticScript[i],"g");
		} else {
			reg = new RegExp(LatinPhoneticScript[i],"ig");
		}
		text = text.replace(reg,CyrillicPhoneticScript[i]);
	}
	return text;
};


/**
 * R2LGui class provides interface functions
 *
 * @ctor
 * Constructs R2LGui class.
 */
function R2LGui() {
/**
 * Convertor object. Used to perform the transliteration.
 * 
 * @type Convertor
 */
 this.convertor = new Convertor();
};


/**
 * R2LGui window startup function. If user had selected some text on the current page,
 * it will automatically be pasted into the translation window and translated.
 * @tparam String dir Direction of conversion, i.e. "lat2rus" or "rus2lat".
 */
R2LGui.prototype.Start = function(dir) {
	var selection = document.commandDispatcher.focusedWindow.getSelection().toString();
	if (selection.length == 0)
	{
		var node = document.commandDispatcher.focusedElement;
		if (node && 'selectionStart' in node && node.selectionStart != node.selectionEnd)
		{
			var offsetStart = Math.min(node.selectionStart, node.selectionEnd);
			var offsetEnd   = Math.max(node.selectionStart, node.selectionEnd);
			selection = node.value.substr(offsetStart, offsetEnd-offsetStart);
		}
	}
	if (selection.length > 0) {
		var result = new String("");
		switch (dir) {
			case "rus2lat":
				result = this.convertor.Rus2LatText(selection);
				break;
			case "lat2rus":
				result = this.convertor.Lat2RusText(selection);
				break;
		}
		window.openDialog("chrome://rus2lat/content/result.xul","result","chrome",selection,result);
	} else {
		window.openDialog("chrome://rus2lat/content/result.xul","result","chrome","","");
        }
};

/**
 * R2LGui window init function.
 */
R2LGui.prototype.Init = function() {
	document.getElementById("source_text").value=window.arguments[0];
	document.getElementById("result_text").value=window.arguments[1];
};

/**
 * R2LGui window translitirate function.
 * @tparam String dir Direction of conversion, i.e. "lat2rus" or "rus2lat".
 */
R2LGui.prototype.Convert = function(dir) {
	source=document.getElementById("source_text").value;
	var result = new String("");
	var source_label = new String("Source:");
	var result_label = new String("Result:");
	switch (dir) {
	case "rus2lat":
		source_label = new String("Rus:");
		result_label = new String("Lat:");
		result = this.convertor.Rus2LatText(source);
		break;
	case "lat2rus":
		source_label = new String("Lat:");
		result_label = new String("Rus:");
		result = this.convertor.Lat2RusText(source);
		break;
	}
	document.getElementById("result_text").value=result;
};

/**
 * "Static" R2LGui Start function.
 * @tparam String dir Direction of conversion, i.e. "lat2rus" or "rus2lat".
 */
function gStart(dir) {
	var g = new R2LGui();
	g.Start(dir);
}

/**
 * "Static" R2LGui Init function.
 */
function gInit() {
	var g = new R2LGui();
	g.Init();
}

/**
 * "Static" R2LGui Convert function.
 * @tparam String dir Direction of conversion, i.e. "lat2rus" or "rus2lat".
 */
function gConvert(dir) {
	var g = new R2LGui();
	g.Convert(dir);
}
